//import crypto from 'crypto'
import { globalPassword } from './pwgen';
import { errorMessage } from './generalStore';
let password: string;
let salt: Uint8Array;
let ciphertext: BufferSource;
let iv: Uint8Array;
globalPassword.subscribe(value => {
  password = value;
})

/*
Fetch the contents of the "message" textbox, and encode it
in a form we can use for the encrypt operation.
*/
function getStringEncoding(string: string) {
  let enc = new TextEncoder();
  return enc.encode(string);
}
/*
Get some key material to use as input to the deriveKey method.
The key material is a password supplied by the user.
*/
function getKeyMaterial() {
    let enc = new TextEncoder();
    return window.crypto.subtle.importKey(
      "raw", 
      enc.encode(password), 
      {name: "PBKDF2"}, 
      false, 
      ["deriveBits", "deriveKey"]
    );
}
/*
Given some key material and some random salt
derive an AES-GCM key using PBKDF2.
Using the Password itself as salt isn't really a great idea, but this is not a hyper secure implementation yet
*/
function getKey(keyMaterial: CryptoKey) {
  let enc = new TextEncoder();
  return window.crypto.subtle.deriveKey(
    {
      "name": "PBKDF2",
      salt: enc.encode(password), 
      "iterations": 100000,
      "hash": "SHA-256"
    },
    keyMaterial,
    { "name": "AES-GCM", "length": 256},
    true,
    [ "encrypt", "decrypt" ]
  );
}

/*
Derive a key from a password supplied by the user, and use the key
to encrypt the message.
Update the "ciphertextValue" box with a representation of part of
the ciphertext.
*/
export async function encryptFile(file: File) {
  let keyMaterial = await getKeyMaterial();
  let key = await getKey(keyMaterial);
  iv = getStringEncoding(password).slice(0,12);
  const fileReader = new FileReader();
  let byteArray = await file.arrayBuffer();

  ciphertext = await window.crypto.subtle.encrypt(
    {
      name: "AES-GCM",
      iv: iv
    },
    key,
    byteArray
  );
  const base64Encoded = btoa(String.fromCharCode(... new Uint8Array(ciphertext)));
  return base64Encoded
}

/*
Derive a key from a password supplied by the user, and use the key
to decrypt the ciphertext.
If the ciphertext was decrypted successfully,
update the "decryptedValue" box with the decrypted value.
If there was an error decrypting,
update the "decryptedValue" box with an error message.
*/
export async function decryptFileContent(base64Encoded: string) {
  //prepare cryptography
  let keyMaterial = await getKeyMaterial();
  console.log(keyMaterial)
  let key = await getKey(keyMaterial);
  iv = getStringEncoding(password).slice(0,12);
  
  //Remove anything form the Base64 String that isn't base64
  const cleanString = base64Encoded.replace(/[^A-Za-z0-9+/=]/g, '');

  // create Uint8 Array from base64 string
  let encryptedString = atob(cleanString)
  const encryptedContent = new Uint8Array(encryptedString.length)
  for (let i = 0; i < encryptedString.length; i++) {
      encryptedContent[i] = encryptedString.charCodeAt(i); // Populate the array with the decoded data
    }
  const encryptedContentBuffer: ArrayBuffer = encryptedContent.buffer;

  try {
    let decrypted = await window.crypto.subtle.decrypt(
      {
        name: "AES-GCM",
        iv: iv      
      },
      key,
      encryptedContentBuffer
    );
    return new Uint8Array(decrypted);
  } catch (e) {
    console.log(e)
    errorMessage.set(`Can not decrypt file: ${e}`)

  }
}
// Hacked together version of https://github.com/mdn/dom-examples/blob/main/web-crypto/derive-key/pbkdf2.jshttps://gist.github.com/ChaoLiangSuper/0e13f77712b68682f0d8ebabb2d63aa8